import logging
from ctypes import byref, c_uint8
from typing import Any, Dict, NamedTuple

from .core import PSP, get_psp_exc_msg
from .exc import (
    PSPError,
    PSPInvalid,
    PSPNotSupport,
)
from .lmbinc import (
    ERR_Success,
    URMODE_RS232,
    URMODE_RS422,
    URMODE_RS485,
    URTERM_OFF,
    URTERM_ON,
)
from .sdk_dll import DLL

logger = logging.getLogger(__name__)

SUPPORTED_1_COM = ("LEB-7242",)
SUPPORTED_2_COM = ("LEB-2680", "LEC-7230")
UNSUPPORTED_COM = ("LEC-2290", "NCA-2510", "V3S", "V6S",)

MODES = ("Loopback", "RS-232", "RS-485", "RS-422")
TERMS = ("Disabled", "Enabled", "----", "-----")


class COMPortInfoModel(NamedTuple):
    """To store COM port information."""
    num: int
    mode: int
    mode_str: str
    termination: bool
    termination_str: str

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dict."""
        return dict(self._asdict())


class COMPort:
    """
    Communication Port.

    :param int num: COM port number
    :raises TypeError: The input parameters type error.
    :raises PSPInvalid: The input parameter is out of range.
    :raises PSPNotSupport: This function is not supported.
    :raises NotImplementedError: It has not been verified to run on this platform.
    """

    def __init__(self, num: int) -> None:
        self._version = DLL().get_version()
        self._num = num
        # Check type.
        if not isinstance(num, int):
            raise TypeError("'num' type must be int")
        # Check value.
        if self._version.platform_id in SUPPORTED_1_COM:
            if num != 1:
                raise PSPInvalid("'num' can only be set to (1) on this platform")
            # Check import for LEC-7242.
            if self._version.platform_id == "LEB-7242":
                try:
                    from portio import ioperm
                except ImportError:
                    raise RuntimeError(
                        "Install lannerpsp with 'lec7242' extra in order to use COM port."
                    )
        elif self._version.platform_id in SUPPORTED_2_COM:
            if not 1 <= num <= 2:
                raise PSPInvalid("'num' can only be set to (1~2) on this platform")
        elif self._version.platform_id in UNSUPPORTED_COM:
            raise PSPNotSupport("Not supported on this platform")
        else:
            raise NotImplementedError

    def get_info(self) -> COMPortInfoModel:
        """
        Get COM port information.

        Example:

        .. code-block:: pycon

            >>> com1 = COMPort(1)
            >>> com1_info = com1.get_info()
            >>> com1_info.num
            1
            >>> com1_info.mode
            485
            >>> com1_info.mode_str
            'RS-485'
            >>> com1_info.termination
            True
            >>> com1_info.termination_str
            'Enabled'
            >>> com1_info.to_dict()
            {'num': 1, 'mode': 485, 'mode_str': 'RS-485', 'termination': True, 'termination_str': 'Enabled'}

        :return: COM port information
        :rtype: COMPortInfoModel
        :raises PSPError: General PSP functional error.
        """
        if self._version.platform_id not in ("LEC-7230",):
            raise PSPNotSupport("Not support on this platform")
        b_mode = c_uint8()
        b_term = c_uint8()
        mode_mapping = {URMODE_RS232: 232, URMODE_RS422: 422, URMODE_RS485: 485}
        termination_mapping = {URTERM_ON: True, URTERM_OFF: False}
        with PSP() as psp:
            i_ret = psp.lib.LMB_ODM_GetUartMode(self._num, byref(b_mode))
            msg = get_psp_exc_msg("LMB_ODM_GetUartMode", i_ret)
            if i_ret != ERR_Success:
                raise PSPError(msg)
            i_ret = psp.lib.LMB_ODM_TermStat(self._num, byref(b_term))
            msg = get_psp_exc_msg("LMB_ODM_TermStat", i_ret)
            if i_ret != ERR_Success:
                raise PSPError(msg)
        return COMPortInfoModel(num=self._num,
                                mode=mode_mapping[b_mode.value],
                                mode_str=MODES[b_mode.value],
                                termination=termination_mapping[b_term.value + 1],
                                termination_str=TERMS[b_term.value])

    def set_mode(self, mode: int) -> None:
        """
        Set COM port mode to RS-232, RS-422 or RS-485.

        Example:

        .. code-block:: pycon

            >>> com1 = COMPort(1)
            >>> com1.set_mode(232)

        :param int mode: 232/422/485
        :raises PermissionError: if not running as root user
        :raises TypeError: The input parameters type error.
        :raises PSPInvalid: The input parameter is out of range.
        :raises PSPError: General PSP functional error.
        """
        if self._version.platform_id == "LEB-7242":
            from .gpio_config_tool import GPIOConfigTool
            GPIOConfigTool().set_com1_mode(mode)
        elif self._version.platform_id in ("LEB-2680", "LEC-7230"):
            self._set_mode(mode)
        else:
            raise PSPNotSupport("Not support on this platform")

    def set_termination(self, enable: bool) -> None:
        """
        Set RS-422/RS-485 termination.

        Example:

        .. code-block:: pycon

            >>> com1 = COMPort(1)
            >>> com1.set_termination(False)

        :param bool enable: set :data:`True` to enable, otherwise :data:`False`
        :raises PermissionError: if not running as root user
        :raises TypeError: The input parameters type error.
        :raises PSPError: General PSP functional error.
        """
        if self._version.platform_id == "LEB-7242":
            from .gpio_config_tool import GPIOConfigTool
            GPIOConfigTool().set_com1_termination(enable)
        elif self._version.platform_id in ("LEB-2680", "LEC-7230"):
            self._set_termination(enable)
        else:
            raise PSPNotSupport("Not support on this platform")

    def _set_mode(self, mode: int) -> None:
        """
        For LEC-7230.

        :raises TypeError: The input parameters type error.
        :raises PSPInvalid: The input parameter is out of range.
        :raises PSPError: General PSP functional error.
        """
        # Check type.
        if not isinstance(mode, int):
            raise TypeError("'mode' type must be int")
        # Check value.
        if mode not in (232, 422, 485):
            raise PSPInvalid("'mode' value must be 232 or 422 or 485")
        # Run.
        mode_mapping = {232: URMODE_RS232, 422: URMODE_RS422, 485: URMODE_RS485}
        b_mode = c_uint8(mode_mapping[mode])
        with PSP() as psp:
            i_ret = psp.lib.LMB_ODM_SetUartMode(self._num, b_mode)
        msg = get_psp_exc_msg("LMB_ODM_SetUartMode", i_ret)
        if i_ret == ERR_Success:
            logger.debug(f"set com port {self._num:d} mode {MODES[mode_mapping[mode]]}")
        else:
            raise PSPError(msg)

    def _set_termination(self, enable: bool) -> None:
        """
        For LEC-7230.

        :param bool enable: set :data:`True` to enable, otherwise :data:`False`
        :raises TypeError: The input parameters type error.
        :raises PSPError: General PSP functional error.
        """
        # Check type.
        if not isinstance(enable, bool):
            raise TypeError("'enable' type must be bool")
        # Run.
        termination_mapping = {True: URTERM_ON, False: URTERM_OFF}
        b_term = c_uint8(termination_mapping[enable])
        with PSP() as psp:
            i_ret = psp.lib.LMB_ODM_Termination(self._num, b_term.value - 1)
        msg = get_psp_exc_msg("LMB_ODM_Termination", i_ret)
        if i_ret == ERR_Success:
            logger.debug(f"set com port {self._num:d} termination {TERMS[termination_mapping[enable] - 1]}")
        else:
            raise PSPError(msg)



"""
Platform: NCA-2510
PSP version: 2.3.1
OS version:
- Ubuntu 18.04: All pass
SDK:
- sdk_bios: Done.
- sdk_dll: Done.
- sdk_eep: Not implement yet, don't know how to use this SDK.
- sdk_gpio: Done.
- sdk_hwm: Done.
- sdk_lbp: Not implement yet, don't know how to use this SDK.
- sdk_lcm: Done, except UART type device.
- sdk_sled: Done.
- sdk_swr: Done.
- sdk_wdt: Done.
"""
from time import sleep

import pytest

from lannerpsp import *

DELAY_TIME = 2  # Seconds.


class TestPSP:

    def test_init(self):
        with PSP():
            pass


class TestDLL:
    dll = DLL()

    def test_get_version(self):
        version = self.dll.get_version()
        assert version.dll_major == 2
        assert version.dll_minor == 3
        assert version.dll_build == 1
        assert version.platform_id == "NCA-2510"
        assert version.board_major == 1
        assert version.board_minor == 1
        assert version.board_build == 0

    def test_get_bios_id(self):
        assert self.dll.get_bios_id() == '*LIID NCA-2510B BIOS V2.02 "08/09/2018"'


class TestGPIO:
    # TODO: Tooling is required.
    pass


class TestHWM:
    hwm = HWM()

    def test_list_supported_sensors(self):
        supported_sensors = self.hwm.list_supported_sensors()
        assert sorted([s.sid for s in supported_sensors]) == sorted([4, 5, 41, 45, 46, 48, 49, 50, 51, 73, 75])
        assert supported_sensors[0].sid == 4
        assert supported_sensors[0].name == "HWMID_TEMP_SYS1"
        assert supported_sensors[1].sid == 5
        assert supported_sensors[1].name == "HWMID_TEMP_SYS2"
        assert supported_sensors[2].sid == 41
        assert supported_sensors[2].name == "HWMID_VCORE_CPU1"
        assert supported_sensors[3].sid == 45
        assert supported_sensors[3].name == "HWMID_VOLT_P12V"
        assert supported_sensors[4].sid == 46
        assert supported_sensors[4].name == "HWMID_VOLT_P5V"
        assert supported_sensors[5].sid == 48
        assert supported_sensors[5].name == "HWMID_VOLT_P5VSB"
        assert supported_sensors[6].sid == 49
        assert supported_sensors[6].name == "HWMID_VOLT_P3V3SB"
        assert supported_sensors[7].sid == 50
        assert supported_sensors[7].name == "HWMID_VOLT_VBAT"
        assert supported_sensors[8].sid == 51
        assert supported_sensors[8].name == "HWMID_VOLT_DDRCH1"
        assert supported_sensors[9].sid == 73
        assert supported_sensors[9].name == "HWMID_RPM_Fan1A"
        assert supported_sensors[10].sid == 75
        assert supported_sensors[10].name == "HWMID_RPM_Fan2A"


class TestLCM:
    lcm = LCM()

    def test_set_backlight(self):
        self.lcm.set_backlight(False)
        sleep(DELAY_TIME)
        self.lcm.set_backlight(True)
        sleep(DELAY_TIME)

    def test_integration(self):
        self.lcm.clear()
        self.lcm.write("Hello Kitty!")
        sleep(DELAY_TIME)
        self.lcm.set_cursor(2, 5)
        sleep(DELAY_TIME)
        self.lcm.write("UFO handsome")
        sleep(DELAY_TIME)
        self.lcm.clear()
        sleep(DELAY_TIME)


class TestSystemLED:
    system_led = SystemLED()

    def test_green(self):
        self.system_led.green()
        sleep(DELAY_TIME)

    def test_red(self):
        self.system_led.red()
        sleep(DELAY_TIME)

    def test_off(self):
        self.system_led.off()
        sleep(DELAY_TIME)


class TestSWR:
    swr = SWR()


class TestWDT:
    wdt = WDT()

    def test_get_info(self):
        wdt_info = self.wdt.get_info()
        assert wdt_info.type == "SuperIO"
        assert wdt_info.max_count == 255
        assert wdt_info.is_minute_support is True

    def test_enable(self):
        self.wdt.enable(10, 2)

    def test_reset(self):
        self.wdt.reset()

    def test_disable(self):
        self.wdt.disable()

    def test_config_out_of_range(self):
        with pytest.raises(PSPInvalid):
            self.wdt.config(256)
        with pytest.raises(PSPInvalid):
            self.wdt.config(10, 3)

    def test_integration(self):
        with pytest.raises(PSPError):
            self.wdt.reset()
        self.wdt.enable(200)
        sleep(DELAY_TIME)
        self.wdt.reset()
        sleep(DELAY_TIME)
        self.wdt.disable()



import logging
from ctypes import byref, c_uint8, CFUNCTYPE
from time import sleep, time
from typing import Optional, Union

from .core import PSP, get_psp_exc_msg
from .exc import (
    PSPError,
    PSPInvalid,
    PSPNotSupport,
)
from .lmbinc import (
    ERR_NotSupport,
    ERR_Success,
    IntrusionMsg,
)
from .sdk_dll import DLL

logger = logging.getLogger(__name__)

SUPPORTED_PLATFORMS = ("LEB-7242", "NCA-2510",)
UNSUPPORTED_PLATFORMS = ("LEB-2680", "LEC-2290", "LEC-7230", "V3S", "V6S",)


class SWR:
    """
    Software Reset Button.

    :param bool check_platform:
        Set to :data:`True` to check if the platform supports this feature.
        Defaults to :data:`False` for better compatibility.
    :raises PSPNotSupport: This function is not supported
        (when ``check_platform`` is set to :data:`True`).
    :raises NotImplementedError: It has not been verified to run on this platform
        (when ``check_platform`` is set to :data:`True`).
    """

    def __init__(self, check_platform: bool = False) -> None:
        self._version = DLL().get_version()
        if not check_platform:
            return
        if self._version.platform_id in SUPPORTED_PLATFORMS:
            pass
        elif self._version.platform_id in UNSUPPORTED_PLATFORMS:
            raise PSPNotSupport("Not supported on this platform")
        else:
            raise NotImplementedError

    def get_status(self) -> int:
        """
        Get software reset button status.

        Example:

        .. code-block:: pycon

            >>> swr = SWR()
            >>> swr.get_status()
            1

        :return: software reset button status
        :rtype: int
        :raises PSPNotSupport: This function is not supported.
        :raises PSPError: General PSP functional error.
        """
        ub_read = c_uint8()
        with PSP() as psp:
            i_ret = psp.lib.LMB_SWR_GetStatus(byref(ub_read))
        msg = get_psp_exc_msg("LMB_SWR_GetStatus", i_ret)
        if i_ret == ERR_Success:
            return ub_read.value
        elif i_ret == ERR_NotSupport:
            raise PSPNotSupport(msg)
        else:
            raise PSPError(msg)

    @classmethod
    def _callback(cls, stu_intrusion_msg: IntrusionMsg) -> None:
        """Callback function for :func:`exec_callback`."""
        print(f"SWR Item = {stu_intrusion_msg.udw_occur_item:04X}, "
              f"Status = {stu_intrusion_msg.udw_status:04X}, "
              f"time is {stu_intrusion_msg.stu_time.uw_year:04d}/"
              f"{stu_intrusion_msg.stu_time.ub_month:02d}/"
              f"{stu_intrusion_msg.stu_time.ub_day:02d} "
              f"{stu_intrusion_msg.stu_time.ub_hour:02d}:"
              f"{stu_intrusion_msg.stu_time.ub_minute:02d}:"
              f"{stu_intrusion_msg.stu_time.ub_second:02d}")

    def exec_callback(self) -> None:
        """
        Use callback function to detect software reset button status (default 10 seconds).

        Example:

        .. code-block:: pycon

            >>> swr = SWR()
            >>> swr.exec_callback()
            ----> hook Software-Reset button Callback OK <----
            ===> wait about 10 second time <===
            SWR Item = 0001, Status = 0001, time is 2022/08/01 17:45:12
            SWR Item = 0001, Status = 0000, time is 2022/08/01 17:45:13
            SWR Item = 0001, Status = 0001, time is 2022/08/01 17:45:14
            SWR Item = 0001, Status = 0000, time is 2022/08/01 17:45:14
            SWR Item = 0001, Status = 0001, time is 2022/08/01 17:45:15
            SWR Item = 0001, Status = 0000, time is 2022/08/01 17:45:16
            ----> disabled Software-Reset button Callback hook <----
        """
        c_callback = CFUNCTYPE(None, IntrusionMsg)
        p_callback = c_callback(self._callback)
        with PSP() as psp:
            i_ret = psp.lib.LMB_SWR_IntrCallback(p_callback, 150)
            if i_ret != ERR_Success:
                print("-----> hook Software-Reset button callback failure <-------")
                return
            print("----> hook Software-Reset button Callback OK <----")
            print("===> wait about 10 second time <===")
            sleep(10)
            i_ret = psp.lib.LMB_SWR_IntrCallback(None, 150)
            if i_ret == ERR_Success:
                print("----> disabled Software-Reset button Callback hook <----")

    def test(self, secs: int = 5) -> None:
        """
        For testing (default 5 seconds delay).

        Example:

        .. code-block:: pycon

            >>> swr = SWR()
            >>> swr.test(5)
            >>> swr.test()
            ===> wait 5 seconds for Software Reset Button trigger .......
            5. 4. 3. 2. 1. 0.
            Software/Reset button not detected ! --> ALARM

        .. code-block:: pycon

            >>> swr.test()
            ===> wait 5 seconds for Software Reset Button trigger .......
            5. 4.
            Software/Reset button pressed ! --> OK

        :param int secs: seconds for test
        :raises TypeError: The input parameters type error.
        :raises ValueError: The input parameters value error.
        """
        # Check type.
        if not isinstance(secs, int):
            raise TypeError("'secs' type must be int")
        # Check value.
        if secs <= 0:
            raise ValueError("'secs' value must be > 0")
        # Run.
        ub_read = c_uint8()
        dw_cnt = 0
        index = int(secs)
        i_time = int(secs)
        print(f"===> wait {secs:d} seconds for Software Reset Button trigger .......")
        with PSP() as psp:
            while True:
                if dw_cnt % 10 == 0:
                    print(f"{index}. ", end="", flush=True)
                    index -= 1
                psp.lib.LMB_SWR_GetStatus(byref(ub_read))
                if ub_read.value == 1:
                    break
                dw_cnt += 1
                if dw_cnt > i_time * 10:
                    break
                sleep(0.1)
        print()
        if ub_read.value == 1:
            print("Software/Reset button pressed ! --> OK")
        else:
            print("\033[1;31mSoftware/Reset button not detected ! --> ALARM\033[0m")

    @property
    def is_pressed(self) -> bool:
        """
        Returns :data:`True` if the device is currently active and :data:`False` otherwise.

        Example:

        .. code-block:: pycon

            >>> swr = SWR()
            >>> swr.is_pressed
            True

        :return: if is pressed or not
        :rtype: bool
        :raises PSPNotSupport: This function is not supported.
        :raises PSPError: General PSP functional error.
        """
        return bool(self.get_status())

    def wait_for_press(self, timeout: Optional[Union[float, int]] = None) -> None:
        """
        Pause the script until the device is activated, or the timeout is reached.

        Example:

        .. code-block:: pycon

            >>> swr = SWR()
            >>> swr.wait_for_press()

        :type timeout: float or int or None
        :param timeout: Number of seconds to wait before proceeding.
            If this is :data:`None` (the default), then wait indefinitely until the device is active.
        :raises TypeError: The input parameters type error.
        :raises PSPInvalid: The input parameters value error.
        :raises PSPNotSupport: This function is not supported.
        :raises PSPError: General PSP functional error.
        """
        if timeout is not None:
            # Check type.
            if not isinstance(timeout, (float, int)):
                raise TypeError("'timeout' type must be float or int or None")
            # Check value.
            if timeout <= 0:
                raise PSPInvalid("'timeout' value must be > 0")
        # Run.
        start_time = time()
        while True:
            if self.is_pressed:
                break
            if timeout and time() - start_time >= timeout:
                break
            sleep(0.1)

    def wait_for_release(self, timeout: Optional[Union[float, int]] = None) -> None:
        """
        Pause the script until the device is deactivated, or the timeout is reached.

        Example:

        .. code-block:: pycon

            >>> swr = SWR()
            >>> swr.wait_for_release()

        :type timeout: float or int or None
        :param timeout: Number of seconds to wait before proceeding.
            If this is :data:`None` (the default), then wait indefinitely until the device is active.
        :raises TypeError: The input parameters type error.
        :raises PSPInvalid: The input parameters value error.
        :raises PSPNotSupport: This function is not supported.
        :raises PSPError: General PSP functional error.
        """
        if timeout is not None:
            # Check type.
            if not isinstance(timeout, (float, int)):
                raise TypeError("'timeout' type must be float or int or None")
            # Check value.
            if timeout <= 0:
                raise PSPInvalid("'timeout' value must be > 0")
        # Run.
        start_time = time()
        while True:
            if not self.is_pressed:
                break
            if timeout and time() - start_time >= timeout:
                break
            sleep(0.1)



from lannerpsp import COMPort

com1 = COMPort(1)

# Set COM port mode to RS-485.
com1.set_mode(485)

# Get COM port information.
com1_info = com1.get_info()

print(com1_info.mode)  # It should print 485.
print(com1_info.mode_str)  # It should print 'RS-485'.



import logging
from ctypes import byref, c_uint32

from .core import PSP, get_psp_exc_msg
from .exc import (
    PSPError,
    PSPInvalid,
    PSPNotSupport,
)
from .lmbinc import (
    ERR_Invalid,
    ERR_Success,
)
from .sdk_dll import DLL

logger = logging.getLogger(__name__)

SUPPORTED_PLATFORMS = ("LEB-7242",)
UNSUPPORTED_PLATFORMS = ("LEC-7230", "NCA-2510", "V3S", "V6S",)


class RFM:
    """
    Radio Frequency Module.

    :param bool check_platform:
        Set to :data:`True` to check if the platform supports this feature.
        Defaults to :data:`False` for better compatibility.
    :raises PSPNotSupport: This function is not supported
        (when ``check_platform`` is set to :data:`True`).
    :raises NotImplementedError: It has not been verified to run on this platform
        (when ``check_platform`` is set to :data:`True`).
    """

    def __init__(self, check_platform: bool = False) -> None:
        self._version = DLL().get_version()
        if not check_platform:
            return
        if self._version.platform_id in SUPPORTED_PLATFORMS:
            pass
        elif self._version.platform_id in UNSUPPORTED_PLATFORMS:
            raise PSPNotSupport("Not supported on this platform")
        else:
            raise NotImplementedError

    def get_power_status(self) -> int:
        """
        Get module power status.

        bit 0 means M.2 module, bit 1 means mPCIe module

        0: power off, 1: power on

        - 0 (00): mPCIe -> off, M.2 -> off
        - 1 (01): mPCIe -> off, M.2 -> on
        - 2 (10): mPCIe -> on,  M.2 -> off
        - 3 (11): mPCIe -> on,  M.2 -> on

        Example:

        .. code-block:: pycon

            >>> rfm = RFM()
            >>> power_status = rfm.get_power_status()
            3

        :return: module power status
        :rtype: int
        :raises PSPError: General PSP functional error.
        """
        udw_reg = c_uint32(0)
        with PSP() as psp:
            i_ret = psp.lib.LMB_RFM_GetModule(byref(udw_reg))
        msg = get_psp_exc_msg("LMB_RFM_GetModule", i_ret)
        if i_ret == ERR_Success:
            logger.debug(f"get module power status {udw_reg.value:x}")
            return udw_reg.value
        else:
            raise PSPError(msg)

    def set_power_status(self, value: int) -> None:
        """
        Set module power status.

        bit 0 means M.2 module, bit 1 means mPCIe module

        0: power off, 1: power on

        - 0 (00): mPCIe -> off, M.2 -> off
        - 1 (01): mPCIe -> off, M.2 -> on
        - 2 (10): mPCIe -> on,  M.2 -> off
        - 3 (11): mPCIe -> on,  M.2 -> on

        Example:

        .. code-block:: pycon

            >>> rfm = RFM()
            >>> rfm.set_power_status(1)

        :param int value: module power status 0 ~ 3
        :raises TypeError: The input parameters type error.
        :raises PSPInvalid: The input parameter is out of range.
        :raises PSPError: General PSP functional error.
        """
        # Check type.
        if not isinstance(value, int):
            raise TypeError("'value' type must be int")
        # Check value has been done by the PSP.
        # Run.
        with PSP() as psp:
            i_ret = psp.lib.LMB_RFM_SetModule(value)
        msg = get_psp_exc_msg("LMB_RFM_SetModule", i_ret)
        if i_ret == ERR_Success:
            logger.debug(f"set module power status {value:d}")
        elif i_ret == ERR_Invalid:
            raise PSPInvalid(msg)
        else:
            raise PSPError(msg)

    def get_sim_status(self) -> int:
        """
        Get SIM card status.

        bit 0 means M.2 module, bit 1 means mPCIe module

        0: first sim, 1: second sim

        - 0 (00): mPCIe -> first sim (SIM3),  M.2 -> first sim (SIM1)
        - 1 (01): mPCIe -> first sim (SIM3),  M.2 -> second sim (SIM2)
        - 2 (10): mPCIe -> second sim (SIM4), M.2 -> first sim (SIM1)
        - 3 (11): mPCIe -> second sim (SIM4), M.2 -> second sim (SIM2)

        Example:

        .. code-block:: pycon

            >>> rfm = RFM()
            >>> sim_status = rfm.get_sim_status()
            0

        :return: SIM card status
        :rtype: int
        :raises PSPError: General PSP functional error.
        """
        udw_reg = c_uint32(0)
        with PSP() as psp:
            i_ret = psp.lib.LMB_RFM_GetSIM(byref(udw_reg))
        msg = get_psp_exc_msg("LMB_RFM_GetSIM", i_ret)
        if i_ret == ERR_Success:
            logger.debug(f"get sim card status {udw_reg.value:x}")
            return udw_reg.value
        else:
            raise PSPError(msg)

    def set_sim_status(self, value: int) -> None:
        """
        Set SIM card status.

        bit 0 means M.2 module, bit 1 means mPCIe module

        0: first sim, 1: second sim

        - 0 (00): mPCIe -> first sim (SIM3),  M.2 -> first sim (SIM1)
        - 1 (01): mPCIe -> first sim (SIM3),  M.2 -> second sim (SIM2)
        - 2 (10): mPCIe -> second sim (SIM4), M.2 -> first sim (SIM1)
        - 3 (11): mPCIe -> second sim (SIM4), M.2 -> second sim (SIM2)

        Example:

        .. code-block:: pycon

            >>> rfm = RFM()
            >>> rfm.set_sim_status(2)

        :param int value: SIM card status 0 ~ 3
        :raises TypeError: The input parameters type error.
        :raises PSPInvalid: The input parameter is out of range.
        :raises PSPError: General PSP functional error.
        """
        # Check type.
        if not isinstance(value, int):
            raise TypeError("'value' type must be int")
        # Check value has been done by the PSP.
        # Run.
        with PSP() as psp:
            i_ret = psp.lib.LMB_RFM_SetSIM(value)
        msg = get_psp_exc_msg("LMB_RFM_SetSIM", i_ret)
        if i_ret == ERR_Success:
            logger.debug(f"set sim card status {value:d}")
        elif i_ret == ERR_Invalid:
            raise PSPInvalid(msg)
        else:
            raise PSPError(msg)



from enum import auto, Enum, unique

HWM_DISPLAY_NAME_MAPPING = {
    # Temperature area
    "HWMID_TEMP_CPU1": "CPU 1 temperature",
    "HWMID_TEMP_CPU2": "CPU 2 temperature",
    "HWMID_TEMP_CPU3": "CPU 3 temperature",
    "HWMID_TEMP_CPU4": "CPU 4 temperature",
    "HWMID_TEMP_SYS1": "SYS 1 temperature",
    "HWMID_TEMP_SYS2": "SYS 2 temperature",
    "HWMID_TEMP_SYS3": "SYS 3 temperature",
    "HWMID_TEMP_SYS4": "SYS 4 temperature",
    "HWMID_TEMP_PCH": "PCH temperature",
    # CPU1 DIMM Temperature
    "HWMID_TEMP_DIMMP1A0": "CPU 1 DIMM A0 temperature",
    "HWMID_TEMP_DIMMP1A1": "CPU 1 DIMM A1 temperature",
    "HWMID_TEMP_DIMMP1B0": "CPU 1 DIMM B0 temperature",
    "HWMID_TEMP_DIMMP1B1": "CPU 1 DIMM B1 temperature",
    "HWMID_TEMP_DIMMP1C0": "CPU 1 DIMM C0 temperature",
    "HWMID_TEMP_DIMMP1C1": "CPU 1 DIMM C1 temperature",
    "HWMID_TEMP_DIMMP1D0": "CPU 1 DIMM D0 temperature",
    "HWMID_TEMP_DIMMP1D1": "CPU 1 DIMM D1 temperature",
    "HWMID_TEMP_DIMMP1E0": "CPU 1 DIMM E0 temperature",
    "HWMID_TEMP_DIMMP1E1": "CPU 1 DIMM E1 temperature",
    "HWMID_TEMP_DIMMP1F0": "CPU 1 DIMM F0 temperature",
    "HWMID_TEMP_DIMMP1F1": "CPU 1 DIMM F1 temperature",
    "HWMID_TEMP_DIMMP1G0": "CPU 1 DIMM G0 temperature",
    "HWMID_TEMP_DIMMP1G1": "CPU 1 DIMM G1 temperature",
    "HWMID_TEMP_DIMMP1H0": "CPU 1 DIMM H0 temperature",
    "HWMID_TEMP_DIMMP1H1": "CPU 1 DIMM H1 temperature",
    # CPU2 DIMM Temperature
    "HWMID_TEMP_DIMMP2A0": "CPU 2 DIMM A0 temperature",
    "HWMID_TEMP_DIMMP2A1": "CPU 2 DIMM A1 temperature",
    "HWMID_TEMP_DIMMP2B0": "CPU 2 DIMM B0 temperature",
    "HWMID_TEMP_DIMMP2B1": "CPU 2 DIMM B1 temperature",
    "HWMID_TEMP_DIMMP2C0": "CPU 2 DIMM C0 temperature",
    "HWMID_TEMP_DIMMP2C1": "CPU 2 DIMM C1 temperature",
    "HWMID_TEMP_DIMMP2D0": "CPU 2 DIMM D0 temperature",
    "HWMID_TEMP_DIMMP2D1": "CPU 2 DIMM D1 temperature",
    "HWMID_TEMP_DIMMP2E0": "CPU 2 DIMM E0 temperature",
    "HWMID_TEMP_DIMMP2E1": "CPU 2 DIMM E1 temperature",
    "HWMID_TEMP_DIMMP2F0": "CPU 2 DIMM F0 temperature",
    "HWMID_TEMP_DIMMP2F1": "CPU 2 DIMM F1 temperature",
    "HWMID_TEMP_DIMMP2G0": "CPU 2 DIMM G0 temperature",
    "HWMID_TEMP_DIMMP2G1": "CPU 2 DIMM G1 temperature",
    "HWMID_TEMP_DIMMP2H0": "CPU 2 DIMM H0 temperature",
    "HWMID_TEMP_DIMMP2H1": "CPU 2 DIMM H1 temperature",
    # Voltage area
    "HWMID_VCORE_CPU1": "CPU 1 Vcore",
    "HWMID_VCORE_CPU2": "CPU 2 Vcore",
    "HWMID_VCORE_CPU3": "CPU 3 Vcore",
    "HWMID_VCORE_CPU4": "CPU 4 Vcore",
    "HWMID_VOLT_P12V": "12V",
    "HWMID_VOLT_P5V": "5V",
    "HWMID_VOLT_P3V3": "3.3V",
    "HWMID_VOLT_P5VSB": "Standby 5V",
    "HWMID_VOLT_P3V3SB": "Standby 3.3V",
    "HWMID_VOLT_VBAT": "Battery",
    "HWMID_VOLT_DDRCH1": "DDR channel 1",
    "HWMID_VOLT_DDRCH2": "DDR channel 2",
    "HWMID_VOLT_DDRCH3": "DDR channel 3",
    "HWMID_VOLT_DDRCH4": "DDR channel 4",
    "HWMID_VOLT_DDRCH5": "DDR channel 5",
    "HWMID_VOLT_DDRCH6": "DDR channel 6",
    "HWMID_VOLT_DDRCH7": "DDR channel 7",
    "HWMID_VOLT_DDRCH8": "DDR channel 8",
    "HWMID_VOLT_PVNN": "VNN",
    "HWMID_VOLT_P1V05": "1.05V",
    "HWMID_VOLT_P1V8": "1.8V",
    "HWMID_VOLT_PVCCIO_CPU1": "CPU 1 VCCIO",
    "HWMID_VOLT_PVCCIO_CPU2": "CPU 2 VCCIO",
    "HWMID_VOLT_PVCCIO_CPU3": "CPU 3 VCCIO",
    "HWMID_VOLT_PVCCIO_CPU4": "CPU 4 VCCIO",
    "HWMID_VOLT_PVCCSA_CPU1": "CPU 1 VCCSA",
    "HWMID_VOLT_PVCCSA_CPU2": "CPU 2 VCCSA",
    "HWMID_VOLT_PVCCSA_CPU3": "CPU 3 VCCSA",
    "HWMID_VOLT_PVCCSA_CPU4": "CPU 4 VCCSA",
    # Fan area
    "HWMID_RPM_FanCpu1": "CPU 1 fan speed",
    "HWMID_RPM_FanCpu2": "CPU 2 fan speed",
    "HWMID_RPM_FanSys1": "SYS 1 fan speed",
    "HWMID_RPM_FanSys2": "SYS 2 fan speed",
    "HWMID_RPM_Fan1A": "Fan 1A speed",
    "HWMID_RPM_Fan1B": "Fan 1B speed",
    "HWMID_RPM_Fan2A": "Fan 2A speed",
    "HWMID_RPM_Fan2B": "Fan 2B speed",
    "HWMID_RPM_Fan3A": "Fan 3A speed",
    "HWMID_RPM_Fan3B": "Fan 3B speed",
    "HWMID_RPM_Fan4A": "Fan 4A speed",
    "HWMID_RPM_Fan4B": "Fan 4B speed",
    "HWMID_RPM_Fan5A": "Fan 5A speed",
    "HWMID_RPM_Fan5B": "Fan 5B speed",
    "HWMID_RPM_Fan6A": "Fan 6A speed",
    "HWMID_RPM_Fan6B": "Fan 6B speed",
    "HWMID_RPM_Fan7A": "Fan 7A speed",
    "HWMID_RPM_Fan7B": "Fan 7B speed",
    "HWMID_RPM_Fan8A": "Fan 8A speed",
    "HWMID_RPM_Fan8B": "Fan 8B speed",
    "HWMID_RPM_Fan9A": "Fan 9A speed",
    "HWMID_RPM_Fan9B": "Fan 9B speed",
    "HWMID_RPM_Fan10A": "Fan 10A speed",
    "HWMID_RPM_Fan10B": "Fan 10B speed",
    # Power supply area
    "HWMID_PSU1_STATUS": "PSU 1 status",
    "HWMID_PSU1_VOLTIN": "PSU 1 Vin",
    "HWMID_PSU1_VOLTOUT": "PSU 1 Vout",
    "HWMID_PSU1_CURRENTIN": "PSU 1 Iin",
    "HWMID_PSU1_CURRENTOUT": "PSU 1 Iout",
    "HWMID_PSU1_POWERIN": "PSU 1 Pin",
    "HWMID_PSU1_POWEROUT": "PSU 1 Pout",
    "HWMID_PSU1_FAN1": "PSU 1 fan 1 speed",
    "HWMID_PSU1_FAN2": "PSU 1 fan 2 speed",
    "HWMID_PSU1_TEMP1": "PSU 1 temperature 1",
    "HWMID_PSU1_TEMP2": "PSU 1 temperature 2",
    "HWMID_PSU2_STATUS": "PSU 2 status",
    "HWMID_PSU2_VOLTIN": "PSU 2 Vin",
    "HWMID_PSU2_VOLTOUT": "PSU 2 Vout",
    "HWMID_PSU2_CURRENTIN": "PSU 2 Iin",
    "HWMID_PSU2_CURRENTOUT": "PSU 2 Iout",
    "HWMID_PSU2_POWERIN": "PSU 2 Pin",
    "HWMID_PSU2_POWEROUT": "PSU 2 Pout",
    "HWMID_PSU2_FAN1": "PSU 2 fan 1 speed",
    "HWMID_PSU2_FAN2": "PSU 2 fan 2 speed",
    "HWMID_PSU2_TEMP1": "PSU 2 temperature 1",
    "HWMID_PSU2_TEMP2": "PSU 2 temperature 2",
    # add here for new items
    #
    "HWMID_VOLT_VCCGT": "VCC GT",
}


@unique
class HWMSensorItemV23(Enum):
    # Temperature area
    HWMID_TEMP_CPU1 = 0
    HWMID_TEMP_CPU2 = auto()
    HWMID_TEMP_CPU3 = auto()
    HWMID_TEMP_CPU4 = auto()
    HWMID_TEMP_SYS1 = auto()
    HWMID_TEMP_SYS2 = auto()
    HWMID_TEMP_SYS3 = auto()
    HWMID_TEMP_SYS4 = auto()
    HWMID_TEMP_PCH = auto()
    # CPU1 DIMM Temperature
    HWMID_TEMP_DIMMP1A0 = auto()
    HWMID_TEMP_DIMMP1A1 = auto()
    HWMID_TEMP_DIMMP1B0 = auto()
    HWMID_TEMP_DIMMP1B1 = auto()
    HWMID_TEMP_DIMMP1C0 = auto()
    HWMID_TEMP_DIMMP1C1 = auto()
    HWMID_TEMP_DIMMP1D0 = auto()
    HWMID_TEMP_DIMMP1D1 = auto()
    HWMID_TEMP_DIMMP1E0 = auto()
    HWMID_TEMP_DIMMP1E1 = auto()
    HWMID_TEMP_DIMMP1F0 = auto()
    HWMID_TEMP_DIMMP1F1 = auto()
    HWMID_TEMP_DIMMP1G0 = auto()
    HWMID_TEMP_DIMMP1G1 = auto()
    HWMID_TEMP_DIMMP1H0 = auto()
    HWMID_TEMP_DIMMP1H1 = auto()
    # CPU2 DIMM Temperature
    HWMID_TEMP_DIMMP2A0 = auto()
    HWMID_TEMP_DIMMP2A1 = auto()
    HWMID_TEMP_DIMMP2B0 = auto()
    HWMID_TEMP_DIMMP2B1 = auto()
    HWMID_TEMP_DIMMP2C0 = auto()
    HWMID_TEMP_DIMMP2C1 = auto()
    HWMID_TEMP_DIMMP2D0 = auto()
    HWMID_TEMP_DIMMP2D1 = auto()
    HWMID_TEMP_DIMMP2E0 = auto()
    HWMID_TEMP_DIMMP2E1 = auto()
    HWMID_TEMP_DIMMP2F0 = auto()
    HWMID_TEMP_DIMMP2F1 = auto()
    HWMID_TEMP_DIMMP2G0 = auto()
    HWMID_TEMP_DIMMP2G1 = auto()
    HWMID_TEMP_DIMMP2H0 = auto()
    HWMID_TEMP_DIMMP2H1 = auto()
    # Voltage area
    HWMID_VCORE_CPU1 = auto()
    HWMID_VCORE_CPU2 = auto()
    HWMID_VCORE_CPU3 = auto()
    HWMID_VCORE_CPU4 = auto()
    HWMID_VOLT_P12V = auto()
    HWMID_VOLT_P5V = auto()
    HWMID_VOLT_P3V3 = auto()
    HWMID_VOLT_P5VSB = auto()
    HWMID_VOLT_P3V3SB = auto()
    HWMID_VOLT_VBAT = auto()
    HWMID_VOLT_DDRCH1 = auto()
    HWMID_VOLT_DDRCH2 = auto()
    HWMID_VOLT_DDRCH3 = auto()
    HWMID_VOLT_DDRCH4 = auto()
    HWMID_VOLT_DDRCH5 = auto()
    HWMID_VOLT_DDRCH6 = auto()
    HWMID_VOLT_DDRCH7 = auto()
    HWMID_VOLT_DDRCH8 = auto()
    HWMID_VOLT_PVNN = auto()
    HWMID_VOLT_P1V05 = auto()
    HWMID_VOLT_PVCCIO_CPU1 = auto()
    HWMID_VOLT_PVCCIO_CPU2 = auto()
    HWMID_VOLT_PVCCIO_CPU3 = auto()
    HWMID_VOLT_PVCCIO_CPU4 = auto()
    HWMID_VOLT_PVCCSA_CPU1 = auto()
    HWMID_VOLT_PVCCSA_CPU2 = auto()
    HWMID_VOLT_PVCCSA_CPU3 = auto()
    HWMID_VOLT_PVCCSA_CPU4 = auto()
    # Fan area
    HWMID_RPM_FanCpu1 = auto()
    HWMID_RPM_FanCpu2 = auto()
    HWMID_RPM_FanSys1 = auto()
    HWMID_RPM_FanSys2 = auto()
    HWMID_RPM_Fan1A = auto()
    HWMID_RPM_Fan1B = auto()
    HWMID_RPM_Fan2A = auto()
    HWMID_RPM_Fan2B = auto()
    HWMID_RPM_Fan3A = auto()
    HWMID_RPM_Fan3B = auto()
    HWMID_RPM_Fan4A = auto()
    HWMID_RPM_Fan4B = auto()
    HWMID_RPM_Fan5A = auto()
    HWMID_RPM_Fan5B = auto()
    HWMID_RPM_Fan6A = auto()
    HWMID_RPM_Fan6B = auto()
    HWMID_RPM_Fan7A = auto()
    HWMID_RPM_Fan7B = auto()
    HWMID_RPM_Fan8A = auto()
    HWMID_RPM_Fan8B = auto()
    HWMID_RPM_Fan9A = auto()
    HWMID_RPM_Fan9B = auto()
    HWMID_RPM_Fan10A = auto()
    HWMID_RPM_Fan10B = auto()
    # Power supply area
    HWMID_PSU1_STATUS = auto()
    HWMID_PSU1_VOLTIN = auto()
    HWMID_PSU1_VOLTOUT = auto()
    HWMID_PSU1_CURRENTIN = auto()
    HWMID_PSU1_CURRENTOUT = auto()
    HWMID_PSU1_POWERIN = auto()
    HWMID_PSU1_POWEROUT = auto()
    HWMID_PSU1_FAN1 = auto()
    HWMID_PSU1_FAN2 = auto()
    HWMID_PSU1_TEMP1 = auto()
    HWMID_PSU1_TEMP2 = auto()
    HWMID_PSU2_STATUS = auto()
    HWMID_PSU2_VOLTIN = auto()
    HWMID_PSU2_VOLTOUT = auto()
    HWMID_PSU2_CURRENTIN = auto()
    HWMID_PSU2_CURRENTOUT = auto()
    HWMID_PSU2_POWERIN = auto()
    HWMID_PSU2_POWEROUT = auto()
    HWMID_PSU2_FAN1 = auto()
    HWMID_PSU2_FAN2 = auto()
    HWMID_PSU2_TEMP1 = auto()
    HWMID_PSU2_TEMP2 = auto()
    # add here for new items
    #
    HWMID_TOTAL = auto()


@unique
class HWMSensorItemV30(Enum):
    # Temperature area
    HWMID_TEMP_CPU1 = 0
    HWMID_TEMP_CPU2 = auto()
    HWMID_TEMP_CPU3 = auto()
    HWMID_TEMP_CPU4 = auto()
    HWMID_TEMP_SYS1 = auto()
    HWMID_TEMP_SYS2 = auto()
    HWMID_TEMP_SYS3 = auto()
    HWMID_TEMP_SYS4 = auto()
    HWMID_TEMP_PCH = auto()
    # CPU1 DIMM Temperature
    HWMID_TEMP_DIMMP1A0 = auto()
    HWMID_TEMP_DIMMP1A1 = auto()
    HWMID_TEMP_DIMMP1B0 = auto()
    HWMID_TEMP_DIMMP1B1 = auto()
    HWMID_TEMP_DIMMP1C0 = auto()
    HWMID_TEMP_DIMMP1C1 = auto()
    HWMID_TEMP_DIMMP1D0 = auto()
    HWMID_TEMP_DIMMP1D1 = auto()
    HWMID_TEMP_DIMMP1E0 = auto()
    HWMID_TEMP_DIMMP1E1 = auto()
    HWMID_TEMP_DIMMP1F0 = auto()
    HWMID_TEMP_DIMMP1F1 = auto()
    HWMID_TEMP_DIMMP1G0 = auto()
    HWMID_TEMP_DIMMP1G1 = auto()
    HWMID_TEMP_DIMMP1H0 = auto()
    HWMID_TEMP_DIMMP1H1 = auto()
    # CPU2 DIMM Temperature
    HWMID_TEMP_DIMMP2A0 = auto()
    HWMID_TEMP_DIMMP2A1 = auto()
    HWMID_TEMP_DIMMP2B0 = auto()
    HWMID_TEMP_DIMMP2B1 = auto()
    HWMID_TEMP_DIMMP2C0 = auto()
    HWMID_TEMP_DIMMP2C1 = auto()
    HWMID_TEMP_DIMMP2D0 = auto()
    HWMID_TEMP_DIMMP2D1 = auto()
    HWMID_TEMP_DIMMP2E0 = auto()
    HWMID_TEMP_DIMMP2E1 = auto()
    HWMID_TEMP_DIMMP2F0 = auto()
    HWMID_TEMP_DIMMP2F1 = auto()
    HWMID_TEMP_DIMMP2G0 = auto()
    HWMID_TEMP_DIMMP2G1 = auto()
    HWMID_TEMP_DIMMP2H0 = auto()
    HWMID_TEMP_DIMMP2H1 = auto()
    # Voltage area
    HWMID_VCORE_CPU1 = auto()
    HWMID_VCORE_CPU2 = auto()
    HWMID_VCORE_CPU3 = auto()
    HWMID_VCORE_CPU4 = auto()
    HWMID_VOLT_P12V = auto()
    HWMID_VOLT_P5V = auto()
    HWMID_VOLT_P3V3 = auto()
    HWMID_VOLT_P5VSB = auto()
    HWMID_VOLT_P3V3SB = auto()
    HWMID_VOLT_VBAT = auto()
    HWMID_VOLT_DDRCH1 = auto()
    HWMID_VOLT_DDRCH2 = auto()
    HWMID_VOLT_DDRCH3 = auto()
    HWMID_VOLT_DDRCH4 = auto()
    HWMID_VOLT_DDRCH5 = auto()
    HWMID_VOLT_DDRCH6 = auto()
    HWMID_VOLT_DDRCH7 = auto()
    HWMID_VOLT_DDRCH8 = auto()
    HWMID_VOLT_PVNN = auto()
    HWMID_VOLT_P1V05 = auto()
    HWMID_VOLT_P1V8 = auto()
    HWMID_VOLT_PVCCIO_CPU1 = auto()
    HWMID_VOLT_PVCCIO_CPU2 = auto()
    HWMID_VOLT_PVCCIO_CPU3 = auto()
    HWMID_VOLT_PVCCIO_CPU4 = auto()
    HWMID_VOLT_PVCCSA_CPU1 = auto()
    HWMID_VOLT_PVCCSA_CPU2 = auto()
    HWMID_VOLT_PVCCSA_CPU3 = auto()
    HWMID_VOLT_PVCCSA_CPU4 = auto()
    # Fan area
    HWMID_RPM_FanCpu1 = auto()
    HWMID_RPM_FanCpu2 = auto()
    HWMID_RPM_FanSys1 = auto()
    HWMID_RPM_FanSys2 = auto()
    HWMID_RPM_Fan1A = auto()
    HWMID_RPM_Fan1B = auto()
    HWMID_RPM_Fan2A = auto()
    HWMID_RPM_Fan2B = auto()
    HWMID_RPM_Fan3A = auto()
    HWMID_RPM_Fan3B = auto()
    HWMID_RPM_Fan4A = auto()
    HWMID_RPM_Fan4B = auto()
    HWMID_RPM_Fan5A = auto()
    HWMID_RPM_Fan5B = auto()
    HWMID_RPM_Fan6A = auto()
    HWMID_RPM_Fan6B = auto()
    HWMID_RPM_Fan7A = auto()
    HWMID_RPM_Fan7B = auto()
    HWMID_RPM_Fan8A = auto()
    HWMID_RPM_Fan8B = auto()
    HWMID_RPM_Fan9A = auto()
    HWMID_RPM_Fan9B = auto()
    HWMID_RPM_Fan10A = auto()
    HWMID_RPM_Fan10B = auto()
    # Power supply area
    HWMID_PSU1_STATUS = auto()
    HWMID_PSU1_VOLTIN = auto()
    HWMID_PSU1_VOLTOUT = auto()
    HWMID_PSU1_CURRENTIN = auto()
    HWMID_PSU1_CURRENTOUT = auto()
    HWMID_PSU1_POWERIN = auto()
    HWMID_PSU1_POWEROUT = auto()
    HWMID_PSU1_FAN1 = auto()
    HWMID_PSU1_FAN2 = auto()
    HWMID_PSU1_TEMP1 = auto()
    HWMID_PSU1_TEMP2 = auto()
    HWMID_PSU2_STATUS = auto()
    HWMID_PSU2_VOLTIN = auto()
    HWMID_PSU2_VOLTOUT = auto()
    HWMID_PSU2_CURRENTIN = auto()
    HWMID_PSU2_CURRENTOUT = auto()
    HWMID_PSU2_POWERIN = auto()
    HWMID_PSU2_POWEROUT = auto()
    HWMID_PSU2_FAN1 = auto()
    HWMID_PSU2_FAN2 = auto()
    HWMID_PSU2_TEMP1 = auto()
    HWMID_PSU2_TEMP2 = auto()
    # add here for new items
    #
    HWMID_TOTAL = auto()



import logging
from ctypes import c_char_p

from .core import PSP, get_psp_exc_msg
from .exc import (
    PSPError,
    PSPNotExist,
    PSPNotSupport,
)
from .lmbinc import (
    ERR_NotExist,
    ERR_NotSupport,
    ERR_Success,
)
from .sdk_dll import DLL

logger = logging.getLogger(__name__)

SUPPORTED_PLATFORMS = ("V3S", "V6S",)
UNSUPPORTED_PLATFORMS = ("LEB-2680", "LEB-7242", "LEC-2290", "LEC-7230", "NCA-2510",)

DEFAULT_GPS_PORT = "/dev/ttyS1"


class GPS:
    """
    Global Positioning System.

    :param bool check_platform:
        Set to :data:`True` to check if the platform supports this feature.
        Defaults to :data:`False` for better compatibility.
    :raises PSPNotSupport: This function is not supported
        (when ``check_platform`` is set to :data:`True`).
    :raises NotImplementedError: It has not been verified to run on this platform
        (when ``check_platform`` is set to :data:`True`).
    """

    def __init__(self, check_platform: bool = False) -> None:
        self._version = DLL().get_version()
        if not check_platform:
            return
        if self._version.platform_id in SUPPORTED_PLATFORMS:
            pass
        elif self._version.platform_id in UNSUPPORTED_PLATFORMS:
            raise PSPNotSupport("Not supported on this platform")
        else:
            raise NotImplementedError

    def search_port(self) -> str:
        """
        Search GPS port.

        Example:

        .. code-block:: pycon

            >>> gps = GPS()
            >>> gps.search_port()
            '/dev/ttyS3'

        :return: the GPS port on the platform
        :rtype: str
        :raises PSPNotSupport: This function is not supported.
        :raises PSPNotExist: The device does not exist.
        :raises PSPError: General PSP functional error.
        """
        str_gps_port = c_char_p(DEFAULT_GPS_PORT.encode())
        with PSP() as psp:
            i_ret = psp.lib.LMB_GPS_SearchPort(str_gps_port)
        msg = get_psp_exc_msg("LMB_GPS_SearchPort", i_ret)
        if i_ret == ERR_Success:
            gps_port = str_gps_port.value.decode()
            return gps_port
        elif i_ret == ERR_NotSupport:
            raise PSPNotSupport(msg)
        elif i_ret == ERR_NotExist:
            raise PSPNotExist(msg)
        else:
            raise PSPError(msg)



import logging

from .core import PSP, get_psp_exc_msg
from .exc import (
    PSPError,
    PSPInvalid,
    PSPNotOpened,
    PSPNotSupport,
)
from .lmbinc import (
    ERR_Invalid,
    ERR_NotOpened,
    ERR_NotSupport,
    ERR_Success,
)
from .sdk_dll import DLL
from .utils import show_delay

logger = logging.getLogger(__name__)

SUPPORTED_PLATFORMS = ("LEB-7242",)
UNSUPPORTED_PLATFORMS = ("LEB-2680", "LEC-2290", "LEC-7230", "NCA-2510", "V3S", "V6S",)


class LTEStressLED:
    """
    LTE Stress LED.

    :param bool check_platform:
        Set to :data:`True` to check if the platform supports this feature.
        Defaults to :data:`False` for better compatibility.
    :raises PSPNotSupport: This function is not supported
        (when ``check_platform`` is set to :data:`True`).
    :raises NotImplementedError: It has not been verified to run on this platform
        (when ``check_platform`` is set to :data:`True`).
    """

    def __init__(self, check_platform: bool = False) -> None:
        self._version = DLL().get_version()
        if not check_platform:
            return
        if self._version.platform_id in SUPPORTED_PLATFORMS:
            pass
        elif self._version.platform_id in UNSUPPORTED_PLATFORMS:
            raise PSPNotSupport("Not supported on this platform")
        else:
            raise NotImplementedError

    def off(self) -> None:
        """
        Set the LTE stress LED display mode to off.

        Example:

        .. code-block:: pycon

            >>> lte_stress_led = LTEStressLED()
            >>> lte_stress_led.off()

        :raises PSPNotOpened: The library is not ready or opened yet.
        :raises PSPInvalid: The input parameter is out of range.
        :raises PSPNotSupport: This function is not supported.
        :raises PSPError: General PSP functional error.
        """
        with PSP() as psp:
            i_ret = psp.lib.LMB_SLED_SetLteStressLED(-1)
        msg = get_psp_exc_msg("LMB_SLED_SetLteStressLED", i_ret)
        if i_ret == ERR_Success:
            logger.debug("set lte stress led off")
        elif i_ret == ERR_NotOpened:
            raise PSPNotOpened(msg)
        elif i_ret == ERR_Invalid:
            raise PSPInvalid(msg)
        elif i_ret == ERR_NotSupport:
            raise PSPNotSupport(msg)
        else:
            raise PSPError(msg)

    def set_strength(self, percent: int) -> None:
        """
        Set the LTE stress LED signal strength.

        Example:

        .. code-block:: pycon

            >>> lte_stress_led = LTEStressLED()
            >>> lte_stress_led.set_strength(87)

        :param int percent: 0 ~ 100 percent of signal strength
        :raises TypeError: The input parameters type error.
        :raises PSPNotOpened: The library is not ready or opened yet.
        :raises PSPInvalid: The input parameter is out of range.
        :raises PSPNotSupport: This function is not supported.
        :raises PSPError: General PSP functional error.
        """
        # Check type.
        if not isinstance(percent, int):
            raise TypeError("'percent' type must be int")
        # Check value.
        if not 0 <= percent <= 100:
            raise PSPInvalid("'percent' value must be between 0 and 100")
        # Run.
        with PSP() as psp:
            i_ret = psp.lib.LMB_SLED_SetLteStressLED(percent)
        msg = get_psp_exc_msg("LMB_SLED_SetLteStressLED", i_ret)
        if i_ret == ERR_Success:
            logger.debug(f"set lte stress led {percent:d}%")
        elif i_ret == ERR_NotOpened:
            raise PSPNotOpened(msg)
        elif i_ret == ERR_Invalid:
            raise PSPInvalid(msg)
        elif i_ret == ERR_NotSupport:
            raise PSPNotSupport(msg)
        else:
            raise PSPError(msg)

    def test(self, secs: int = 2) -> None:
        """
        For testing (default 2 seconds delay).

        Example:

        .. code-block:: pycon

            >>> lte_stress_led = LTEStressLED()
            >>> lte_stress_led.test(2)
            lte stress led show level 8
            2. 1. 0.
            lte stress led show level 7
            2. 1. 0.
            lte stress led show level 6
            2. 1. 0.
            lte stress led show level 5
            2. 1. 0.
            lte stress led show level 4
            2. 1. 0.
            lte stress led show level 3
            2. 1. 0.
            lte stress led show level 2
            2. 1. 0.
            lte stress led show level 1
            2. 1. 0.
            set lte stress led off

        :param int secs: seconds for test
        :raises TypeError: The input parameters type error.
        :raises ValueError: The input parameters value error.
        """
        # Check type.
        if not isinstance(secs, int):
            raise TypeError("'secs' type must be int")
        # Check value.
        if secs <= 0:
            raise ValueError("'secs' value must be >= 0")
        # Run.
        with PSP() as psp:
            i_ret = psp.lib.LMB_SLED_SetLteStressLED(90)
            if i_ret != ERR_Success:
                msg = get_psp_exc_msg("LMB_SLED_SetLteStressLED", i_ret)
                print(f"\033[1;31m{msg}\033[0m")
            else:
                print("lte stress led show level 8")

            show_delay(secs)

            i_ret = psp.lib.LMB_SLED_SetLteStressLED(78)
            if i_ret != ERR_Success:
                msg = get_psp_exc_msg("LMB_SLED_SetLteStressLED", i_ret)
                print(f"\033[1;31m{msg}\033[0m")
            else:
                print("lte stress led show level 7")

            show_delay(secs)

            i_ret = psp.lib.LMB_SLED_SetLteStressLED(66)
            if i_ret != ERR_Success:
                msg = get_psp_exc_msg("LMB_SLED_SetLteStressLED", i_ret)
                print(f"\033[1;31m{msg}\033[0m")
            else:
                print("lte stress led show level 6")

            show_delay(secs)

            i_ret = psp.lib.LMB_SLED_SetLteStressLED(54)
            if i_ret != ERR_Success:
                msg = get_psp_exc_msg("LMB_SLED_SetLteStressLED", i_ret)
                print(f"\033[1;31m{msg}\033[0m")
            else:
                print("lte stress led show level 5")

            show_delay(secs)

            i_ret = psp.lib.LMB_SLED_SetLteStressLED(42)
            if i_ret != ERR_Success:
                msg = get_psp_exc_msg("LMB_SLED_SetLteStressLED", i_ret)
                print(f"\033[1;31m{msg}\033[0m")
            else:
                print("lte stress led show level 4")

            show_delay(secs)

            i_ret = psp.lib.LMB_SLED_SetLteStressLED(30)
            if i_ret != ERR_Success:
                msg = get_psp_exc_msg("LMB_SLED_SetLteStressLED", i_ret)
                print(f"\033[1;31m{msg}\033[0m")
            else:
                print("lte stress led show level 3")

            show_delay(secs)

            i_ret = psp.lib.LMB_SLED_SetLteStressLED(18)
            if i_ret != ERR_Success:
                msg = get_psp_exc_msg("LMB_SLED_SetLteStressLED", i_ret)
                print(f"\033[1;31m{msg}\033[0m")
            else:
                print("lte stress led show level 2")

            show_delay(secs)

            i_ret = psp.lib.LMB_SLED_SetLteStressLED(6)
            if i_ret != ERR_Success:
                msg = get_psp_exc_msg("LMB_SLED_SetLteStressLED", i_ret)
                print(f"\033[1;31m{msg}\033[0m")
            else:
                print("lte stress led show level 1")

            show_delay(secs)

            i_ret = psp.lib.LMB_SLED_SetLteStressLED(-1)
            if i_ret != ERR_Success:
                msg = get_psp_exc_msg("LMB_SLED_SetLteStressLED", i_ret)
                print(f"\033[1;31m{msg}\033[0m")
            else:
                print("set lte stress led off")



import logging
from ctypes import byref, c_uint32
from math import log2
from typing import Any, Dict, NamedTuple

from .core import PSP, convert_to_bit_array, get_psp_exc_msg
from .exc import (
    PSPError,
    PSPInvalid,
    PSPNotOpened,
    PSPNotSupport,
)
from .lmbinc import (
    DISABLE,
    ENABLE,
    ERR_NotOpened,
    ERR_NotSupport,
    ERR_Success,
)
from .sdk_dll import DLL

logger = logging.getLogger(__name__)

SUPPORTED_PLATFORMS = ("LEB-2680", "LEC-2290", "V3S", "V6S",)
UNSUPPORTED_PLATFORMS = ("LEB-7242", "LEC-7230", "NCA-2510",)


class PoEInfoModel(NamedTuple):
    """To store PoE information."""
    number_of_poe_ports: int
    power_status: Dict[int, bool]

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dict."""
        return dict(self._asdict())


class PoE:
    """
    Power over Ethernet.

    :param int num: LAN port number
    :raises TypeError: The input parameters type error.
    :raises PSPNotSupport: This function is not supported.
    :raises PSPInvalid: The input parameter is out of range.
    :raises PSPError: General PSP functional error.
    """

    def __init__(self, num: int) -> None:
        self._version = DLL().get_version()
        self._num = num
        # Check type.
        if not isinstance(num, int):
            raise TypeError("'num' type must be int")
        # Check value.
        number_of_poe_ports = self._get_supported_ports_count()
        if not 1 <= num <= number_of_poe_ports:
            raise PSPInvalid(f"'num' can only be set to (1~{number_of_poe_ports}) on this platform")

    @classmethod
    def get_info(cls) -> PoEInfoModel:
        """
        Get the PoE information supported by this platform.

        Example:

        .. code-block:: pycon

            >>> poe_info = PoE.get_info()
            >>> poe_info
            PoEInfoModel(number_of_poe_ports=6, power_status={1: True, 2: False, 3: True, 4: True, 5: True, 6: True})
            >>> poe_info.number_of_poe_ports
            6
            >>> poe_info.power_status
            {1: True, 2: False, 3: True, 4: True, 5: True, 6: True}
            >>> poe_info.to_dict()
            {'number_of_poe_ports': 6, 'power_status': {1: True, 2: True, 3: True, 4: True, 5: True, 6: True}}

        :return: The PoE information.
        :rtype: PoEInfoModel
        :raises PSPNotSupport: This function is not supported.
        :raises PSPNotOpened: Device port is not opened yet.
        :raises PSPError: General PSP functional error.
        """
        number_of_poe_ports = cls._get_supported_ports_count()
        # Get the power status of each port.
        udw_status = c_uint32(0)
        with PSP() as psp:
            i_ret = psp.lib.LMB_IGN_GetPoePower(0xFFFFFFFF, byref(udw_status))
            try:
                # Prevent the UART of the MCU from being occupied.
                psp.lib.LMB_IGN_ClosePort()
            except AttributeError:
                pass
        msg = get_psp_exc_msg("LMB_IGN_GetPoePower", i_ret)
        if i_ret == ERR_Success:
            # Convert integer to bit array.
            power_status_list = convert_to_bit_array(udw_status.value)
            # Fill the space with 0 from the beginning of the list.
            power_status_list[0:0] = [0 for _ in range(number_of_poe_ports - len(power_status_list))]
            # Reverse the list so that the port numbers start at 1.
            power_status_list.reverse()
            power_status = {}
            for i in range(number_of_poe_ports):
                power_status[i + 1] = bool(power_status_list[i])
        elif i_ret == ERR_NotOpened:
            raise PSPNotOpened(msg)
        else:
            raise PSPError(msg)
        return PoEInfoModel(
            number_of_poe_ports=number_of_poe_ports,
            power_status=power_status,
        )

    def enable(self) -> None:
        """
        Enable the PoE port power (power on by auto).

        Example:

        .. code-block:: pycon

            >>> poe1 = PoE(1)
            >>> poe1.enable()

        :raises PSPNotOpened: The library is not ready or opened yet.
        :raises PSPError: General PSP functional error.
        """
        with PSP() as psp:
            i_ret = psp.lib.LMB_POE_SetPortPower(self._num, ENABLE)
            try:
                # Prevent the UART of the MCU from being occupied.
                psp.lib.LMB_IGN_ClosePort()
            except AttributeError:
                pass
        msg = get_psp_exc_msg("LMB_POE_SetPortPower", i_ret)
        if i_ret == ERR_Success:
            logger.debug(f"LAN{self._num} port power on by auto")
        elif i_ret == ERR_NotOpened:
            raise PSPNotOpened(msg)
        else:
            raise PSPError(msg)

    def disable(self) -> None:
        """
        Disable the PoE port power (power off).

        Example:

        .. code-block:: pycon

            >>> poe1 = PoE(1)
            >>> poe1.disable()

        :raises PSPNotOpened: The library is not ready or opened yet.
        :raises PSPError: General PSP functional error.
        """
        with PSP() as psp:
            i_ret = psp.lib.LMB_POE_SetPortPower(self._num, DISABLE)
            try:
                # Prevent the UART of the MCU from being occupied.
                psp.lib.LMB_IGN_ClosePort()
            except AttributeError:
                pass
        msg = get_psp_exc_msg("LMB_POE_SetPortPower", i_ret)
        if i_ret == ERR_Success:
            logger.debug(f"LAN{self._num} port power off")
        elif i_ret == ERR_NotOpened:
            raise PSPNotOpened(msg)
        else:
            raise PSPError(msg)

    def get_power_status(self) -> bool:
        """
        Get the PoE port power status.

        Example:

        .. code-block:: pycon

            >>> poe1 = PoE(1)
            >>> poe1.get_power_status()
            True

        :return: PoE LAN port power status. :data:`True` means PoE power is enabled by auto,
            :data:`False` means PoE power is disabled.
        :rtype: bool
        :raises PSPNotOpened: The library is not ready or opened yet.
        :raises PSPError: General PSP functional error.
        """
        udw_status = c_uint32(0)
        with PSP() as psp:
            i_ret = psp.lib.LMB_POE_GetPortStatus(self._num, byref(udw_status))
            try:
                # Prevent the UART of the MCU from being occupied.
                psp.lib.LMB_IGN_ClosePort()
            except AttributeError:
                pass
        msg = get_psp_exc_msg("LMB_POE_GetPortStatus", i_ret)
        if i_ret == ERR_Success:
            return bool(udw_status.value)
        elif i_ret == ERR_NotOpened:
            raise PSPNotOpened(msg)
        else:
            raise PSPError(msg)

    @classmethod
    def _get_supported_ports_count(cls) -> int:
        """
        Get the number of LAN ports that support PoE.

        :raises PSPNotSupport: This function is not supported.
        :raises PSPError: General PSP functional error.
        """
        udw_ports = c_uint32(0)
        with PSP() as psp:
            try:
                i_ret = psp.lib.LMB_POE_QueryDevices(byref(udw_ports))
            except AttributeError:
                raise PSPNotSupport("Not supported on this platform")
            try:
                # Prevent the UART of the MCU from being occupied.
                psp.lib.LMB_IGN_ClosePort()
            except AttributeError:
                pass
        msg = get_psp_exc_msg("LMB_POE_QueryDevices", i_ret)
        if i_ret == ERR_Success:
            logger.debug(f"PoE ports = 0x{udw_ports.value:08X}")
            return int(log2(udw_ports.value + 1))
        elif i_ret == ERR_NotSupport:
            raise PSPNotSupport(msg)
        else:
            raise PSPError(msg)



import logging
from ctypes import cdll, CDLL
from typing import List

from .exc import (
    PSPBoardNotMatch,
    PSPBusyInUses,
    PSPDriverNotLoad,
    PSPError,
    PSPInvalid,
    PSPNotExist,
    PSPNotOpened,
    PSPNotSupport,
)
from .lmbinc import (
    ERR_BoardNotMatch,
    ERR_BusyInUses,
    ERR_DriverNotLoad,
    ERR_Error,
    ERR_IPMI_IDLESTATE,
    ERR_IPMI_WRITESTATE,
    ERR_IPMI_READSTATE,
    ERR_IPMI_IBF0,
    ERR_IPMI_OBF1,
    ERR_Invalid,
    ERR_NotExist,
    ERR_NotOpened,
    ERR_NotSupport,
    ERR_Success,
)
from .utils import is_root

logger = logging.getLogger(__name__)

DEFAULT_LMB_IO_PATH = "/opt/lanner/psp/bin/amd64/lib/liblmbio.so"
DEFAULT_LMB_API_PATH = "/opt/lanner/psp/bin/amd64/lib/liblmbapi.so"


class PSP:
    """
    Lanner Platform Support Package (PSP).

    :raises PermissionError: if not running as root user
    """
    lmb_io_path = DEFAULT_LMB_IO_PATH
    lmb_api_path = DEFAULT_LMB_API_PATH

    def __init__(self) -> None:
        if not is_root():
            raise PermissionError("Please uses root user !!!")
        self._liblmbio = cdll.LoadLibrary(self.lmb_io_path)
        self._liblmbapi = cdll.LoadLibrary(self.lmb_api_path)

    def __enter__(self) -> "PSP":
        """
        Initialize the Lanner common API and board libraries.

        :returns: the :class:`PSP` instance
        :rtype: PSP
        :raises PSPNotExist: Board library is not found.
        :raises PSPBoardNotMatch: Library and M/B do not match.
        :raises PSPError: Initializing library failed.
        """
        i_ret = self._liblmbapi.LMB_DLL_Init()
        msg = get_psp_exc_msg("LMB_DLL_Init", i_ret)
        if i_ret == ERR_Success:
            return self
        msg += " please confirm the API libraries is matched this platform or the lmbiodrv driver was loaded"
        if i_ret == ERR_NotExist:
            raise PSPNotExist(msg)
        elif i_ret == ERR_NotOpened:
            raise PSPNotOpened(msg)
        elif i_ret == ERR_Invalid:
            raise PSPInvalid(msg)
        elif i_ret == ERR_NotSupport:
            raise PSPNotSupport(msg)
        elif i_ret == ERR_BusyInUses:
            raise PSPBusyInUses(msg)
        elif i_ret == ERR_BoardNotMatch:
            raise PSPBoardNotMatch(msg)
        elif i_ret == ERR_DriverNotLoad:
            raise PSPDriverNotLoad(msg)
        else:
            raise PSPError(msg)

    def __exit__(self, exc_type, exc_val, exc_tb) -> bool:
        """
        Release the Lanner common API library.

        .. note::

            This function will auto-release board-level library.

        :raises PSPBusyInUses: Libray is busy or a certain process is in use.
        :raises PSPError: For generic PSP error.
        """
        i_ret = self._liblmbapi.LMB_DLL_DeInit()
        msg = get_psp_exc_msg("LMB_DLL_DeInit", i_ret)
        if i_ret == ERR_Success:
            pass
        elif i_ret == ERR_BusyInUses:
            raise PSPBusyInUses(msg)
        else:
            raise PSPError(msg)
        return False

    @property
    def lib(self) -> CDLL:
        """
        Get the DLL/SO to call C functions directly.

        Example:

        .. code-block:: pycon

            >>> with PSP() as psp:  # Automatically Init() and DeInit().
            ...     psp.lib.LMB_SLED_SetLteStateLED(0)
            ...
            0
        """
        return self._liblmbapi


def get_psp_exc_msg(func_name: str, ret_code: int) -> str:
    """
    Get the message of the execution result of the PSP function
    by ``func_name`` and ``ret_code``.

    :param str func_name: the name of the C function where the error occurred
    :param int ret_code: error code returned by the C function
    :return: error message details
    :rtype: str
    """
    msg = f"{func_name} return code ( 0x{ret_code & 0xFFFFFFFF:08x} --> "
    if ret_code == ERR_Success:
        msg += "OK )"
    elif ret_code == ERR_Error:
        msg += "general error )"
    elif ret_code == ERR_NotExist:
        msg += "device or file not exist )"
    elif ret_code == ERR_NotOpened:
        msg += "library not opened yet )"
    elif ret_code == ERR_Invalid:
        msg += "parameter out of range or invalid )"
    elif ret_code == ERR_NotSupport:
        msg += "hardware or function not support )"
    elif ret_code == ERR_BusyInUses:
        msg += "device is busy now )"
    elif ret_code == ERR_BoardNotMatch:
        msg += "board BIOSID and library not matched )"
    elif ret_code == ERR_DriverNotLoad:
        msg += "the lmbiodrv driver or i2c-dev driver not loading )"
    elif ret_code == ERR_IPMI_IDLESTATE:
        msg += "IPMI idle state error )"
    elif ret_code == ERR_IPMI_WRITESTATE:
        msg += "IPMI write state error )"
    elif ret_code == ERR_IPMI_READSTATE:
        msg += "IPMI read state error )"
    elif ret_code == ERR_IPMI_IBF0:
        msg += "IPMI input wait error )"
    elif ret_code == ERR_IPMI_OBF1:
        msg += "IPMI output wait error )"
    else:
        msg += "unknown error )"
    return msg


def convert_to_bit_array(n: int) -> List[int]:
    """
    Convert int to bit array.

    Example:

    .. code-block:: pycon

        >>> from lannerpsp import *
        >>> gpio = GPIO()
        >>> gpio.get_digital_in()
        12
        >>> convert_to_bit_array(gpio.get_digital_in())
        [1, 1, 0, 0]

    :param int n: An integer to be converted.
    :return: A list of 0 or 1.
    :rtype: typing.List[int]
    """
    return [1 if digit == "1" else 0 for digit in bin(n)[2:]]  # [2:] to chop off the "0b" part



import logging
from ctypes import byref, c_int32, c_uint8, c_uint32
from math import log2
from typing import Any, Dict, NamedTuple

from .core import PSP, get_psp_exc_msg
from .exc import (
    PSPError,
    PSPNotOpened,
    PSPNotSupport,
)
from .lmbinc import (
    ERR_NotOpened,
    ERR_NotSupport,
    ERR_Success,
)
from .sdk_dll import DLL

logger = logging.getLogger(__name__)

SUPPORTED_PLATFORMS = ("LEB-2680", "LEC-2290", "LEC-7230", "NCA-2510", "V3S", "V6S",)
UNSUPPORTED_PLATFORMS = ("LEB-7242",)


class GPIOInfoModel(NamedTuple):
    """To store GPIO information."""
    number_of_di_pins: int
    number_of_do_pins: int

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dict."""
        return dict(self._asdict())


class GPIO:
    """
    General Purpose Input/Output.

    :param bool check_platform:
        Set to :data:`True` to check if the platform supports this feature.
        Defaults to :data:`False` for better compatibility.
    :raises PSPNotSupport: This function is not supported
        (when ``check_platform`` is set to :data:`True`).
    :raises NotImplementedError: It has not been verified to run on this platform
        (when ``check_platform`` is set to :data:`True`).
    """

    def __init__(self, check_platform: bool = False) -> None:
        self._version = DLL().get_version()
        if not check_platform:
            return
        if self._version.platform_id in SUPPORTED_PLATFORMS:
            pass
        elif self._version.platform_id in UNSUPPORTED_PLATFORMS:
            raise PSPNotSupport("Not supported on this platform")
        else:
            raise NotImplementedError

    def get_info(self) -> GPIOInfoModel:
        """
        Get the GPIO information supported by this platform.

        Example:

        .. code-block:: pycon

            >>> gpio = GPIO()
            >>> gpio_info = gpio.get_info()
            >>> gpio_info
            GPIOInfoModel(number_of_di_pins=8, number_of_do_pins=4)
            >>> gpio_info.number_of_di_pins
            8
            >>> gpio_info.number_of_do_pins
            4
            >>> gpio_info.to_dict()
            {'number_of_di_pins': 8, 'number_of_do_pins': 4}

        :return: The GPIO information.
        :rtype: GPIOInfoModel
        :raises PSPNotSupport: This function is not supported.
        :raises PSPNotOpened: Device port is not opened yet.
        :raises PSPError: General PSP functional error.
        """
        if self._version.platform_id in ("LEB-2680",):
            # Use ignition MCU.
            udw_in_pins = c_uint32(0)
            udw_out_pins = c_uint32(0)
            with PSP() as psp:
                i_ret = psp.lib.LMB_IGN_GetDigitalPins(byref(udw_out_pins), byref(udw_in_pins))
                psp.lib.LMB_IGN_ClosePort()  # Prevent the UART of the MCU from being occupied.
            msg = get_psp_exc_msg("LMB_IGN_GetDigitalPins", i_ret)
            if i_ret == ERR_Success:
                return GPIOInfoModel(
                    number_of_di_pins=int(log2(udw_in_pins.value + 1)),
                    number_of_do_pins=int(log2(udw_out_pins.value + 1)),
                )
            elif i_ret == ERR_NotSupport:
                raise PSPNotSupport(msg)
            elif i_ret == ERR_NotOpened:
                raise PSPNotOpened(msg)
            else:
                raise PSPError(msg)
        else:
            ub_in_pins = c_uint8(0)
            ub_out_pins = c_uint8(0)
            with PSP() as psp:
                i_ret = psp.lib.LMB_GPIO_GetInfo(0, byref(ub_in_pins), byref(ub_out_pins))
                try:
                    # Prevent the UART of the MCU from being occupied.
                    psp.lib.LMB_IGN_ClosePort()
                except AttributeError:
                    pass
            msg = get_psp_exc_msg("LMB_GPIO_GetInfo", i_ret)
            if i_ret == ERR_Success:
                return GPIOInfoModel(
                    number_of_di_pins=ub_in_pins.value,
                    number_of_do_pins=ub_out_pins.value,
                )
            elif i_ret == ERR_NotSupport:
                raise PSPNotSupport(msg)
            elif i_ret == ERR_NotOpened:
                raise PSPNotOpened(msg)
            else:
                raise PSPError(msg)

    def get_digital_in(self) -> int:
        """
        Get the GPI/DI status.

        Example:

        .. code-block:: pycon

            >>> gpio = GPIO()
            >>> gpio.get_digital_in()
            12

        :return: GPI/DI status in decimal. When converted to binary, the LSB represents DI_0.
        :rtype: int
        :raises PSPNotSupport: This function is not supported.
        :raises PSPNotOpened: Device port is not opened yet.
        :raises PSPError: General PSP functional error.
        """
        gpio_info = self.get_info()
        udw_dio_stat = c_int32(0)
        if self._version.platform_id in ("LEB-2680",):
            # Use ignition MCU.
            with PSP() as psp:
                i_ret = psp.lib.LMB_IGN_GetDigitalIn(2 ** gpio_info.number_of_di_pins - 1, byref(udw_dio_stat))
                psp.lib.LMB_IGN_ClosePort()  # Prevent the UART of the MCU from being occupied.
            msg = get_psp_exc_msg("LMB_IGN_GetDigitalIn", i_ret)
        else:
            with PSP() as psp:
                i_ret = psp.lib.LMB_GPIO_GpiRead(0, byref(udw_dio_stat))
                try:
                    # Prevent the UART of the MCU from being occupied.
                    psp.lib.LMB_IGN_ClosePort()
                except AttributeError:
                    pass
            msg = get_psp_exc_msg("LMB_GPIO_GpiRead", i_ret)
        if i_ret == ERR_Success:
            logger.debug(f"read DI status: 0x{udw_dio_stat.value:02X}")
            return udw_dio_stat.value
        elif i_ret == ERR_NotSupport:
            raise PSPNotSupport(msg)
        elif i_ret == ERR_NotOpened:
            raise PSPNotOpened(msg)
        else:
            raise PSPError(msg)

    def get_digital_out(self) -> int:
        """
        Get the GPO/DO status.

        Example:

        .. code-block:: pycon

            >>> gpio = GPIO()
            >>> gpio.get_digital_out()
            3

        :return: GPO/DO status in decimal. When converted to binary, the LSB represents DO_0.
        :rtype: int
        :raises PSPNotSupport: This function is not supported.
        :raises PSPNotOpened: Device port is not opened yet.
        :raises PSPError: General PSP functional error.
        """
        gpio_info = self.get_info()
        udw_dio_stat = c_int32(0)
        if self._version.platform_id in ("LEB-2680",):
            # Use ignition MCU.
            with PSP() as psp:
                i_ret = psp.lib.LMB_IGN_GetDigitalOut(2 ** gpio_info.number_of_do_pins - 1, byref(udw_dio_stat))
                psp.lib.LMB_IGN_ClosePort()  # Prevent the UART of the MCU from being occupied.
            msg = get_psp_exc_msg("LMB_IGN_GetDigitalOut", i_ret)
        else:
            with PSP() as psp:
                i_ret = psp.lib.LMB_GPIO_GpoRead(0, byref(udw_dio_stat))
                try:
                    # Prevent the UART of the MCU from being occupied.
                    psp.lib.LMB_IGN_ClosePort()
                except AttributeError:
                    pass
            msg = get_psp_exc_msg("LMB_GPIO_GpoRead", i_ret)
        if i_ret == ERR_Success:
            logger.debug(f"read DO status: 0x{udw_dio_stat.value:02X}")
            return udw_dio_stat.value
        elif i_ret == ERR_NotSupport:
            raise PSPNotSupport(msg)
        elif i_ret == ERR_NotOpened:
            raise PSPNotOpened(msg)
        else:
            raise PSPError(msg)

    def set_digital_out(self, status: int) -> None:
        """
        Set the GPO/DO status.

        Example:

        .. code-block:: pycon

            >>> gpio = GPIO()
            >>> gpio.set_digital_out(0x0A)

        :param int status: GPO/DO status.
            This value can be binary, octal, decimal or hexadecimal with type :class:`int`.
            For example, to write the ``[high, low, high, low]`` signal from **DO_3** to **DO_0**,
            just set this parameter to ``0b1010``, ``0o12``, ``0x0A`` or ``10``.
        :raises PSPNotSupport: This function is not supported.
        :raises PSPNotOpened: Device port is not opened yet.
        :raises PSPError: General PSP functional error.
        """
        if not isinstance(status, int):
            raise TypeError("'status' type must be int")
        gpio_info = self.get_info()
        if self._version.platform_id in ("LEB-2680",):
            # Use ignition MCU.
            with PSP() as psp:
                i_ret = psp.lib.LMB_IGN_SetDigitalOut(2 ** gpio_info.number_of_do_pins - 1, status)
                psp.lib.LMB_IGN_ClosePort()  # Prevent the UART of the MCU from being occupied.
            msg = get_psp_exc_msg("LMB_IGN_SetDigitalOut", i_ret)
        else:
            with PSP() as psp:
                i_ret = psp.lib.LMB_GPIO_GpoWrite(0, status)
                try:
                    # Prevent the UART of the MCU from being occupied.
                    psp.lib.LMB_IGN_ClosePort()
                except AttributeError:
                    pass
            msg = get_psp_exc_msg("LMB_GPIO_GpoWrite", i_ret)
        if i_ret == ERR_Success:
            logger.debug(f"write DI status: {status:d}")
        elif i_ret == ERR_NotSupport:
            raise PSPNotSupport(msg)
        elif i_ret == ERR_NotOpened:
            raise PSPNotOpened(msg)
        else:
            raise PSPError(msg)



"""
Platform: LEC-2290
PSP version: 2.1.2
OS version:
- Debian 11: All pass
SDK:
- sdk_dll: Done.
- sdk_gpio: Done.
- sdk_hwm: Done.
- sdk_ign: Not implement yet, don't know how to use this SDK.
- sdk_poe: Done.
- sdk_wdt: Done.
"""
from time import sleep

import pytest

from lannerpsp import *

DELAY_TIME = 2  # Seconds.


class TestPSP:

    def test_init(self):
        with PSP():
            pass


class TestDLL:
    dll = DLL()

    def test_get_version(self):
        version = self.dll.get_version()
        assert version.dll_major == 2
        assert version.dll_minor == 1
        assert version.dll_build == 2
        assert version.platform_id == "LEC-2290"
        assert version.board_major == 1
        assert version.board_minor == 0
        assert version.board_build == 2

    def test_get_bios_id(self):
        assert self.dll.get_bios_id() == 'LEB-2291B BIOS V2.02 "06/08/2021"'


class TestGPIO:
    # TODO: Tooling is required.
    pass


class TestHWM:
    hwm = HWM()

    def test_list_supported_sensors(self):
        supported_sensors = self.hwm.list_supported_sensors()
        assert sorted([s.sid for s in supported_sensors]) == sorted([
            0, 2, 4, 6, 7, 10, 11, 72,
            225, 226, 227, 228, 229, 230, 232, 233,
        ])
        assert supported_sensors[0].sid == 0
        assert supported_sensors[0].name == "HWMID_TEMP_CPU1"
        assert supported_sensors[1].sid == 2
        assert supported_sensors[1].name == "HWMID_TEMP_SYS1"
        assert supported_sensors[2].sid == 4
        assert supported_sensors[2].name == "HWMID_VCORE_CPU1"
        assert supported_sensors[3].sid == 6
        assert supported_sensors[3].name == "HWMID_VOLT_P12V"
        assert supported_sensors[4].sid == 7
        assert supported_sensors[4].name == "HWMID_VOLT_P5V"
        assert supported_sensors[5].sid == 10
        assert supported_sensors[5].name == "HWMID_VOLT_P3V3SB"
        assert supported_sensors[6].sid == 11
        assert supported_sensors[6].name == "HWMID_VOLT_VBAT"
        assert supported_sensors[7].sid == 72
        assert supported_sensors[7].name == "HWMID_VOLT_VCCGT"


class TestPoE:

    def test_get_supported_ports(self):
        poe_info = PoE.get_info()
        assert poe_info.number_of_poe_ports == 4

    def test_set_all(self):
        for i in range(4):
            poe1 = PoE(i + 1)
            poe1.enable()
            assert poe1.get_power_status() is True
            assert PoE.get_info().power_status[i + 1] is True
            poe1.disable()
            assert poe1.get_power_status() is False
            assert PoE.get_info().power_status[i + 1] is False

    def test_init_out_of_range(self):
        with pytest.raises(PSPInvalid) as e:
            PoE(5)
        assert str(e.value) == "'num' can only be set to (1~4) on this platform"


class TestWDT:
    wdt = WDT()

    def test_get_info(self):
        wdt_info = self.wdt.get_info()
        assert wdt_info.type == "SuperIO"
        assert wdt_info.max_count == 255
        assert wdt_info.is_minute_support is True

    def test_enable(self):
        self.wdt.enable(10, 2)

    def test_reset(self):
        # PSP version 2.1 has a bug in the WDT `reset` function
        # that stops the timer like `disable`.
        self.wdt.reset()

    def test_disable(self):
        self.wdt.disable()

    def test_config_out_of_range(self):
        with pytest.raises(PSPInvalid):
            self.wdt.config(256)
        with pytest.raises(PSPInvalid):
            self.wdt.config(10, 3)

    def test_integration(self):
        with pytest.raises(PSPError):
            self.wdt.reset()
        self.wdt.enable(200)
        sleep(DELAY_TIME)
        self.wdt.reset()
        sleep(DELAY_TIME)
        self.wdt.disable()



import logging
from ctypes import byref
from typing import Any, Dict, NamedTuple

from .core import PSP, get_psp_exc_msg
from .exc import (
    PSPBusyInUses,
    PSPError,
    PSPInvalid,
    PSPNotOpened,
    PSPNotSupport,
)
from .lmbinc import (
    BASE_SECOND,
    BASE_MINUTE,
    ERR_BusyInUses,
    ERR_Invalid,
    ERR_NotOpened,
    ERR_NotSupport,
    ERR_Success,
    WDT_TYPE_SIO,
    WDT_TYPE_TCO,
    WDT_TYPE_UNKNOWN,
    WDTInfo,
)
from .sdk_dll import DLL

logger = logging.getLogger(__name__)

SUPPORTED_PLATFORMS = ("LEB-2680", "LEB-7242", "LEC-2290", "LEC-7230", "NCA-2510", "V3S", "V6S",)
UNSUPPORTED_PLATFORMS = ()


class WDTInfoModel(NamedTuple):
    """To store WDT info values."""
    type: str
    max_count: int
    is_minute_support: bool

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dict."""
        return dict(self._asdict())


class WDT:
    """
    Watch Dog Timer.

    :param bool check_platform:
        Set to :data:`True` to check if the platform supports this feature.
        Defaults to :data:`False` for better compatibility.
    :raises PSPNotSupport: This function is not supported
        (when ``check_platform`` is set to :data:`True`).
    :raises NotImplementedError: It has not been verified to run on this platform
        (when ``check_platform`` is set to :data:`True`).
    """

    def __init__(self, check_platform: bool = False) -> None:
        self._version = DLL().get_version()
        if not check_platform:
            return
        if self._version.platform_id in SUPPORTED_PLATFORMS:
            pass
        elif self._version.platform_id in UNSUPPORTED_PLATFORMS:
            raise PSPNotSupport("Not supported on this platform")
        else:
            raise NotImplementedError

    def get_info(self) -> WDTInfoModel:
        """
        Get Watch Dog Timer information.

        Example:

        .. code-block:: pycon

            >>> wdt = WDT()
            >>> wdt.get_info()
            WDTInfoModel(type='SuperIO', max_count=255, is_minute_support=True)

        :return: WDT information
        :rtype: WDTInfoModel
        :raises PSPNotOpened: The library is not ready or opened yet.
        :raises PSPNotSupport: This platform does not support this function.
        :raises PSPError: General PSP functional error.
        """
        stu_wdt_info = WDTInfo()
        type_mapping = {WDT_TYPE_UNKNOWN: "Unknown", WDT_TYPE_SIO: "SuperIO", WDT_TYPE_TCO: "TCO"}
        with PSP() as psp:
            i_ret = psp.lib.LMB_WDT_QueryInfo(byref(stu_wdt_info))
        msg = get_psp_exc_msg("LMB_WDT_QueryInfo", i_ret)
        if i_ret == ERR_Success:
            return WDTInfoModel(
                type=type_mapping[stu_wdt_info.ub_type],
                max_count=stu_wdt_info.uw_count_max,
                is_minute_support=bool(stu_wdt_info.ub_minute_support),
            )
        elif i_ret == ERR_NotOpened:
            raise PSPNotOpened(msg)
        elif i_ret == ERR_NotSupport:
            raise PSPNotSupport(msg)
        else:
            raise PSPError(msg)

    def config(self, count: int, time_base: int = 1) -> None:
        """
        Configure the Watch Dog Timer for specific time.

        Example for 200 seconds:

        .. code-block:: pycon

            >>> wdt = WDT()
            >>> wdt.config(200)

        Example for 2 minutes:

        .. code-block:: pycon

            >>> wdt = WDT()
            >>> wdt.config(2, 2)

        :param int count:
            The value sets the timer count down.

        :param int time_base:
            The value selects time base. Set ``1`` to select SECOND base,
            Set ``2`` to select MINUTE base. Defaults to ``1``.

        :raises TypeError: The input parameters type error.
        :raises PSPInvalid: Invalid parameter value.
        :raises PSPNotOpened: The library is not ready or opened yet.
        :raises PSPNotSupport: This platform does not support this function.
        :raises PSPBusyInUses: This step is skipped because WDT is already starting now.
        :raises PSPError: General PSP functional error.
        """
        # Check type.
        if not isinstance(count, int):
            raise TypeError("'count' type must be int")
        if not isinstance(time_base, int):
            raise TypeError("'time_base' type must be int")
        # Check value.
        info = self.get_info()
        if not 0 <= count <= info.max_count:
            raise PSPInvalid(f"'count' value must be between 0 and {info.max_count}")
        if time_base not in (BASE_SECOND, BASE_MINUTE):
            raise PSPInvalid(f"'time_base' value must be {BASE_SECOND}"
                             f" for SECOND base or {BASE_MINUTE} for MINUTE base")
        if not info.is_minute_support and time_base == BASE_MINUTE:
            raise PSPInvalid("WDT only support SECOND base")
        # Run.
        time_base_mapping = {BASE_SECOND: "seconds", BASE_MINUTE: "minutes"}
        with PSP() as psp:
            i_ret = psp.lib.LMB_WDT_Config(count, time_base)
        msg = get_psp_exc_msg("LMB_WDT_Config", i_ret)
        if i_ret == ERR_Success:
            logger.debug(f"configure the watchdog timer for {count:d} {time_base_mapping[time_base]}")
        elif i_ret == ERR_Invalid:
            raise PSPInvalid(msg)
        elif i_ret == ERR_NotOpened:
            raise PSPNotOpened(msg)
        elif i_ret == ERR_NotSupport:
            raise PSPNotSupport(msg)
        elif i_ret == ERR_BusyInUses:
            raise PSPBusyInUses(msg)
        else:
            raise PSPError(msg)

    def enable(self, count: int = 0, time_base: int = 1) -> None:
        """
        Configure the Watch Dog Timer for specific time and start the WDT countdown.

        You can :func:`enable` dircetly by a given time:

        .. code-block:: pycon

            >>> wdt = WDT()
            >>> wdt.enable(200)

        Or :func:`config` first then :func:`enable`

        .. code-block:: pycon

            >>> wdt = WDT()
            >>> wdt.config(200)
            >>> wdt.enable()

        :param int count:
            The value sets the timer count down. Defaults to 0.

        :param int time_base:
            The value selects time base. Set ``1`` to select SECOND base,
            Set ``2`` to select MINUTE base. Defaults to ``1``.

        :raises TypeError: The input parameters type error.
        :raises PSPInvalid: Invalid parameter value.
        :raises PSPNotOpened: The library is not ready or opened yet.
        :raises PSPNotSupport: This platform does not support this function.
        :raises PSPBusyInUses: This step is skipped because WDT is already starting now.
        :raises PSPError: General PSP functional error.
        """
        if count != 0:
            self.config(count, time_base)
        with PSP() as psp:
            i_ret = psp.lib.LMB_WDT_Start()
        msg = get_psp_exc_msg("LMB_WDT_Start", i_ret)
        if i_ret == ERR_Success:
            logger.debug("enable watchdog timer")
        elif i_ret == ERR_NotOpened:
            raise PSPNotOpened(msg)
        elif i_ret == ERR_NotSupport:
            raise PSPNotSupport(msg)
        else:
            raise PSPError(msg)

    def disable(self) -> None:
        """
        Stop the WDT countdown.

        Example:

        .. code-block:: pycon

            >>> wdt = WDT()
            >>> wdt.enable(10)
            >>>
            >>> wdt.disable()

        :raises PSPNotOpened: The library is not ready or opened yet.
        :raises PSPNotSupport: This platform does not support this function.
        :raises PSPError: General PSP functional error.
        """
        with PSP() as psp:
            i_ret = psp.lib.LMB_WDT_Stop()
        msg = get_psp_exc_msg("LMB_WDT_Stop", i_ret)
        if i_ret == ERR_Success:
            logger.debug("disable watchdog timer")
        elif i_ret == ERR_NotOpened:
            raise PSPNotOpened(msg)
        elif i_ret == ERR_NotSupport:
            raise PSPNotSupport(msg)
        else:
            raise PSPError(msg)

    def reset(self) -> None:
        """
        Reload the timer and then re-computes it.

        Example:

        .. code-block:: pycon

            >>> wdt = WDT()
            >>> wdt.enable(10)
            >>>
            >>> wdt.reset()

        :raises PSPNotOpened: The library is not ready or opened yet.
        :raises PSPNotSupport: This platform does not support this function.
        :raises PSPError: General PSP functional error.
        """
        with PSP() as psp:
            i_ret = psp.lib.LMB_WDT_Tick()
        msg = get_psp_exc_msg("LMB_WDT_Tick", i_ret)
        if i_ret == ERR_Success:
            logger.debug("reset watchdog timer")
        elif i_ret == ERR_NotOpened:
            raise PSPNotOpened(msg)
        elif i_ret == ERR_NotSupport:
            raise PSPNotSupport(msg)
        else:
            raise PSPError(msg)



from time import sleep

from lannerpsp import LTEStatusLED

led = LTEStatusLED()

led.red()
sleep(2)
led.red_blink()
sleep(2)
led.green()
sleep(2)
led.green_blink()
sleep(2)
led.yellow()
sleep(2)
led.yellow_blink()
sleep(2)
led.off()
